/**
 * @file
 * Right-to-left specific stylesheet for the Dashboard module.
 */

#dashboard div.dashboard-region {
  float: right;
}
#dashboard #disabled-blocks .block, #dashboard .block-placeholder {
  float: right;
  margin: 3px 0 3px 3px;
  padding: 6px 8px 6px 4px;
}
#dashboard .canvas-content a.button {
  margin: 0 10px 0 0;
}
#dashboard .ui-sortable .block h2 {
  background-position: right -39px;
  padding: 0 19px;
}
#dashboard.customize-inactive #disabled-blocks .block:hover h2 {
  background-position: right -39px;
}
#dashboard.customize-inactive .dashboard-region .ui-sortable .block:hover h2 {
  background-position: right -36px;
}
#dashboard div#dashboard_main {
  margin-left: 1%;
  margin-right: 0;
}
