
#permissions td.module {
  font-weight: bold;
}
#permissions td.permission {
  padding-left: 1.5em; /* LTR */
}
#permissions tr.odd .form-item,
#permissions tr.even .form-item {
  white-space: normal;
}
#user-admin-settings fieldset .fieldset-description {
  font-size: 0.85em;
  padding-bottom: .5em;
}

/**
 * Override default textfield float to put the "Add role" button next to
 * the input textfield.
 */
#user-admin-roles td.edit-name {
  clear: both;
}
#user-admin-roles .form-item-name {
  float: left; /* LTR */
  margin-right: 1em; /* LTR */
}

/**
 * Password strength indicator.
 */
.password-strength {
  width: 17em;
  float: right;  /* LTR */
  margin-top: 1.4em;
}
.password-strength-title {
  display: inline;
}
.password-strength-text {
  float: right; /* LTR */
  font-weight: bold;
}
.password-indicator {
  background-color: #C4C4C4;
  height: 0.3em;
  width: 100%;
}
.password-indicator div {
  height: 100%;
  width: 0%;
  background-color: #47C965;
}
input.password-confirm,
input.password-field {
  width: 16em;
  margin-bottom: 0.4em;
}
div.password-confirm {
  float: right;  /* LTR */
  margin-top: 1.5em;
  visibility: hidden;
  width: 17em;
}
div.form-item div.password-suggestions {
  padding: 0.2em 0.5em;
  margin: 0.7em 0;
  width: 38.5em;
  border: 1px solid #B4B4B4;
}
div.password-suggestions ul {
  margin-bottom: 0;
}
.confirm-parent,
.password-parent {
  clear: left; /* LTR */
  margin: 0;
  width: 36.3em;
}

/* Generated by user.module but used by profile.module: */
.profile {
  clear: both;
  margin: 1em 0;
}
.profile .user-picture {
  float: right; /* LTR */
  margin: 0 1em 1em 0; /* LTR */
}
.profile h3 {
  border-bottom: 1px solid #ccc;
}
.profile dl {
  margin: 0 0 1.5em 0;
}
.profile dt {
  margin: 0 0 0.2em 0;
  font-weight: bold;
}
.profile dd {
  margin: 0 0 1em 0;
}
