
div.vertical-tabs {
  margin-left: 0;
  margin-right: 15em;
}
.vertical-tabs ul.vertical-tabs-list {
  margin-left: 0;
  margin-right: -15em;
  float: right;
}
.vertical-tabs ul.vertical-tabs-list li.selected {
  border-left-width: 0;
  border-right-width: 1px;
}
