/**
 * @file
 * Right-to-left styling for the Forum module.
 */

#forum td.forum .icon {
  float: right;
  margin: 0 0 0 9px;
}
.forum-topic-navigation {
  padding: 1em 3em 0 0;
}
.forum-topic-navigation .topic-previous {
  text-align: left;
  float: right;
}
.forum-topic-navigation .topic-next {
  text-align: right;
  float: left;
}
