/**
 * @file
 * Stylesheet specific to right-to-left languages.
 */

div.contextual-links-wrapper {
  left: 5px;
  right: auto;
}
div.contextual-links-wrapper ul.contextual-links {
  -moz-border-radius: 0 4px 4px 4px;
  -webkit-border-top-left-radius: 0;
  -webkit-border-top-right-radius: 4px;
  border-radius: 0 4px 4px 4px;
  left: 0;
  right: auto;
}
a.contextual-links-trigger {
  text-indent: -90px;
}
