
#content {
  overflow: hidden;
}
.form-item-search-block-form {
  width: 50%;
}
.tabs ul.primary,
.region-header .block-menu li a,
.comment-form .form-item {
  zoom: 1;
}
#block-search-form .form-item-search-block-form input {
  width: 67%;
}
.node-teaser {
  border-bottom: 1px solid #d3d7d9;
}
