
tr.taxonomy-term-preview {
  background-color: #EEE;
}
tr.taxonomy-term-divider-top {
  border-bottom: none;
}
tr.taxonomy-term-divider-bottom {
  border-top: 1px dotted #CCC;
}
.taxonomy-term-description {
  margin: 5px 0 20px;
}
