
#aggregator .feed-source .feed-title {
  margin-top: 0;
}
#aggregator .feed-source .feed-image img {
  margin-bottom: 0.75em;
}
#aggregator .feed-source .feed-icon {
  float: right; /* LTR */
  display: block;
}
#aggregator .feed-item {
  margin-bottom: 1.5em;
}
#aggregator .feed-item-title {
  margin-bottom: 0;
  font-size: 1.3em;
}
#aggregator .feed-item-meta,
#aggregator .feed-item-body {
  margin-bottom: 0.5em;
}
#aggregator .feed-item-categories {
  font-size: 0.9em;
}
#aggregator td {
  vertical-align: bottom;
}
#aggregator td.categorize-item {
  white-space: nowrap;
}
#aggregator .categorize-item .news-item .body {
  margin-top: 0;
}
#aggregator .categorize-item h3 {
  margin-bottom: 1em;
  margin-top: 0;
}
