
/**
 * Generic elements
 */
body {
  margin: 0;
  padding: 0;
  background: #edf5fa;
  font: 76%/170% Verdana, sans-serif;
  color: #494949;
}

input {
  font: 1em/100% Verdana, sans-serif;
  color: #494949;
}

textarea,
select {
  font: 1em/160% Verdana, sans-serif;
  color: #494949;
}

h1,
h2,
h3,
h4,
h5,
h6 {
  margin: 0;
  padding: 0;
  font-weight: normal;
  font-family: Helvetica, Arial, sans-serif;
}

h1 {
  font-size: 170%;
}

h2,
#center h1 {
  font-size: 160%;
  line-height: 130%;
}

h3 {
  font-size: 140%;
}

h4 {
  font-size: 130%;
}

h5 {
  font-size: 120%;
}

h6 {
  font-size: 110%;
}

quote,
code,
fieldset {
  margin: .5em 0;
}

code,
pre,
kbd {
  font-size: 115%;
}

p {
  margin: 0.6em 0 1.2em;
  padding: 0;
}

a:link,
a:visited {
  color: #027AC6;
  text-decoration: none;
}

a:hover {
  color: #0062A0;
  text-decoration: underline;
}

a:active,
a.active {
  color: #5895be;
}

hr {
  margin: 0;
  padding: 0;
  border: none;
  height: 1px;
  background: #5294c1;
}

ul,
.block ul,
ol {
  margin: 0.5em 0 1em;
  padding: 0 0 0 1.5em; /* LTR */
}

/* Default to menu leaf bullet for unordered list items. "ul" used here so it can cascade to list items and "li.leaf" to override the system leaf image. */
ul,
ul li.leaf {
  list-style-image: url(images/menu-leaf.gif);
}

/* This is used to cancel the list style image when an ordered list is embedded in an unordered list. */
ol {
  list-style-image: none;
}

ul.menu,
.item-list ul {
  margin: 0.35em 0 0 -0.5em; /* LTR */
}

ol li,
ul li,
ul.menu li,
.item-list ul li,
li.leaf {
  margin: 0.15em 0 0.15em .5em; /* LTR */
  padding-bottom: .1em;
}

ul li.expanded {
  list-style-image: url(images/menu-expanded.gif);
}

ul li.collapsed {
  list-style-image: url(images/menu-collapsed.gif); /* LTR */
}

ul li.leaf a,
ul li.expanded a,
ul li.collapsed a {
  display: block;
}

ul.inline li {
  background: none;
  margin: 0;
  padding: 0 1em 0 0; /* LTR */
}

ol.task-list {
  margin-left: 0; /* LTR */
  list-style-type: none;
  list-style-image: none;
}
ol.task-list li {
  padding: 0.5em 1em 0.5em 2em; /* LTR */
}
ol.task-list li.active {
  background: transparent url(images/task-list.png) no-repeat 3px 50%; /* LTR */
}
ol.task-list li.done {
  color: #393;
  background: transparent url(../../misc/watchdog-ok.png) no-repeat 0px 50%; /* LTR */
}
ol.task-list li.active {
  margin-right: 1em; /* LTR */
}

fieldset ul.clearfix li {
  margin: 0;
  padding: 0;
  background-image: none;
}

dl {
  margin: 0.5em 0 1em 1.5em; /* LTR */
}

dl dt {
}

dl dd {
  margin: 0 0 .5em 1.5em; /* LTR */
}

img,
a img {
  border: none;
}

table {
  margin: 1em 0;
  width: 100%;
}

thead th {
  border-bottom: 2px solid #d3e7f4;
  color: #494949;
  font-weight: bold;
}

th a:link,
th a:visited {
  color: #6f9dbd;
}

td,
th {
  padding: .3em .5em;
}

tr.even,
tr.odd,
tbody th {
  border: solid #d3e7f4;
  border-width: 1px 0;
}

tr.odd,
tr.info {
  background-color: #edf5fa;
}

tr.even {
  background-color: #fff;
}

tr.drag {
  background-color: #ffb;
}

tr.drag-previous {
  background-color: #ffd;
}

tr.odd td.active {
  background-color: #ddecf5;
}

tr.even td.active {
  background-color: #e6f1f7;
}

td.region-title,
td.module,
td.container,
td.category {
  border-top: 1.5em solid #fff;
  border-bottom: 1px solid #b4d7f0;
  background-color: #d4e7f3;
  color: #455067;
  font-weight: bold;
}

tr:first-child td.region-title,
tr:first-child td.module,
tr:first-child td.container,
tr:first-child td.category {
  border-top-width: 0;
}

span.form-required {
  color: #ffae00;
}

.submitted, .description,
.vertical-tab-button .summary {
  font-size: 0.92em;
  color: #898989;
}

.description .more-help-link {
  font-size: 0.92em;
}

.description {
  line-height: 150%;
  margin-bottom: 0.75em;
  color: #898989;
}

.preview {
  margin: .75em 0 .75em;
  padding: .5em 1em;
}

.form-checkboxes,
.form-radios,
.form-checkboxes .form-item,
.form-radios .form-item {
  margin: 0.25em 0;
}

#center form {
  margin-bottom: 2em;
}

.form-actions {
  margin: 2em 0 1em;
}
.form-button,
.form-submit {
  margin: 0 0.5em 0 0; /* LTR */
}

/**
 * Skip link
 */
#skip-link {
  left: 50%;
  margin-left: -5.25em;
  margin-top: 0;
  position: absolute;
  width: auto;
  z-index: 1000;
}
#skip-link a,
#skip-link a:link,
#skip-link a:visited {
  background: #444;
  background: rgba(0, 0, 0, 0.6);
  color: #fff;
  display: block;
  font-size: 0.94em;
  line-height: 1.7;
  margin-top: 1px;
  padding: 2px 10px;
  text-decoration: none;
  -khtml-border-radius: 0 0 2px 2px;
  -moz-border-radius: 0 0 2px 2px;
  -webkit-border-bottom-left-radius: 2px;
  -webkit-border-bottom-right-radius: 2px;
  -webkit-border-top-left-radius: 0;
  -webkit-border-top-right-radius: 0;
  border-radius: 0 0 2px 2px;
}
#skip-link a:hover,
#skip-link a:active,
#skip-link a:focus {
  outline: 0;
}

/**
 * Layout
 */
.region-header {
  min-height: 1em;
  background: #d2e6f3 url(images/bg-navigation.png) repeat-x 50% 100%;
}

.region-header .block {
  display: block;
  margin: 0 1em;
}

.region-header .block-region {
  display: block;
  margin: 0 0.5em 1em;
  padding: 0.5em;
  position: relative;
  top: 0.5em;
}

.region-header * {
  display: inline;
  line-height: 1.5em;
  margin-top: 0;
  margin-bottom: 0;
}

/* Prevent the previous directive from showing the content of script elements in Mozilla browsers. */
.region-header script {
  display: none;
}

.region-header p,
.region-header img {
  margin-top: 0.5em;
}

.region-header h2 {
  margin: 0 1em 0 0; /* LTR */
}

.region-header h3,
.region-header label,
.region-header li {
  margin: 0 1em;
  padding: 0;
  background: none;
}

#wrapper {
  background: #edf5fa url(images/body.png) repeat-x 50% 0;
}

/* Set container widths when we are using the "fixed width" option. */
#wrapper #container {
  width: 560px;
  margin: 0 auto;
  padding: 0 20px;
}

body.one-sidebar #wrapper #container {
  width: 770px;
}

body.two-sidebars #wrapper #container {
  width: 980px;
}

/* Unset container width when we are using the "fluid width" option. */
body.fluid-width #wrapper #container {
  width: auto;
  max-width: 1270px;
}

#wrapper #container #header {
  height: 80px;
}

#wrapper #container #header #logo-floater {
  position: absolute;
}

#branding,
#branding a:link,
#branding a:visited {
  line-height: 120px;
  position: relative;
  z-index: 2;
  white-space: nowrap;
}

#branding span {
  font-weight: bold;
}

#branding img {
  padding-top: 14px;
  padding-right: 20px; /* LTR */
  float: left; /* LTR */
}

/* With 3 columns, require a minimum width of 1000px to ensure there is enough horizontal space. */
body.two-sidebars {
  min-width: 980px;
}
/* With 2 columns, require a minimum width of 800px. */
body.sidebar-first,
body.sidebar-second {
  min-width: 780px;
}

/* We must define 100% width to avoid the body being too narrow for near-empty pages */
#wrapper #container #center {
  float: left; /* LTR */
  width: 100%;
}

/* So we move the #center container over the sidebars to compensate */
body.sidebar-first #center {
  margin-left: -210px; /* LTR */
}
body.sidebar-second #center {
  margin-right: -210px; /* LTR */
}
body.two-sidebars #center {
  margin: 0 -210px;
}

/* And add blanks left and right for the sidebars to fill */
body.sidebar-first #squeeze {
  margin-left: 210px; /* LTR */
}
body.sidebar-second #squeeze {
  margin-right: 210px; /* LTR */
}
body.two-sidebars #squeeze {
  margin: 0 210px;
}

/* We ensure the sidebars are still clickable using z-index */
#wrapper #container .sidebar {
  margin: 60px 0 5em;
  width: 210px;
  float: left; /* LTR */
  z-index: 2;
  position: relative;
}

#wrapper #container .sidebar .block {
  margin: 0 0 1.5em 0;
}

#sidebar-first .block {
  padding: 0 15px 0 0; /* LTR */
}

#sidebar-second .block {
  padding: 0 0 0 15px; /* LTR */
}

.block .content {
  margin: 0.5em 0;
}

#sidebar-first .block-region {
  margin: 0 15px 0 0px; /* LTR */
}

#sidebar-second .block-region {
  margin: 0 0px 0 15px; /* LTR */
}

.block-region {
  padding: 1em;
  background: transparent;
  border: 2px dashed #b4d7f0;
  text-align: center;
  font-size: 1.3em;
}

/* Now we add the backgrounds for the main content shading */
#wrapper #container #center #squeeze {
  background: #fff url(images/bg-content.png) repeat-x 50% 0;
  position: relative;
}

#wrapper #container #center .right-corner {
  background: transparent url(images/bg-content-right.png) no-repeat 100% 0;
  position: relative;
  left: 10px;
}

#wrapper #container #center .right-corner .left-corner {
  padding: 60px 25px 5em 35px;
  background: transparent url(images/bg-content-left.png) no-repeat 0 0;
  margin-left: -10px;
  position: relative;
  left: -10px;
  min-height: 400px;
}

#wrapper #container .region-footer {
  float: none;
  clear: both;
  text-align: center;
  margin: 4em 0 -3em;
  color: #898989;
}

#wrapper #container .breadcrumb {
  position: absolute;
  top: 15px;
  left: 35px; /* LTR */
  z-index: 3;
}

body.sidebar-first .region-footer {
  margin-left: -210px;
}

body.sidebar-second .region-footer {
  margin-right: -210px;
}

body.two-sidebars .region-footer {
  margin: 0 -210px;
}

/**
 * Header
 */
div#branding {
  font-family: Helvetica,Arial,sans-serif;
}

div#branding strong {
   font-weight: normal;
}

#branding,
#branding a:link,
#branding a:visited {
  color: #fff;
  text-shadow: #1659ac 0px 1px 3px;
  font-size: 1.5em;
}

#branding a:hover {
  text-decoration: none;
}

#wrapper #container .breadcrumb {
  font-size: 0.92em;
}

#wrapper #container .breadcrumb,
#wrapper #container .breadcrumb a {
  color: #529ad6;
}

#highlighted {
  padding: 1em;
  background-color: #fff;
  border: 1px solid #e0e5fb;
  margin-bottom: 2em;
}

/**
 * Overlay
 */
#overlay #overlay-tabs li a {
  background: #d9eaf5;
  color: #000;
}
#overlay #overlay-tabs li a:hover,
#overlay #overlay-tabs li a:focus {
  background: #fff;
}
#overlay #overlay-tabs li.active a {
  background: url("images/body.png") repeat-x scroll 50% -58px #edf5fa;
  color: #fff;
}
#overlay-content {
  padding: 1px;
}
#overlay-content #header {
  display: none;
}
#overlay-content #wrapper {
  background-position: 50% -80px;
}


/**
 * Primary navigation
 */
ul.main-menu {
  margin: 0;
  padding: 0;
  float: right; /* LTR */
  position: relative;
  z-index: 4;
}

ul.main-menu li {
  margin: 0;
  padding: 0;
  float: left; /* LTR */
  background-image: none;
}

ul.main-menu li a,
ul.main-menu li a:link,
ul.main-menu li a:visited {
  display: block;
  margin: 0 1em;
  padding: .75em 0 0;
  color: #fff;
  background: transparent url(images/bg-navigation-item.png) no-repeat 50% 0;
}

ul.main-menu li a:hover,
ul.main-menu li a.active {
  color: #fff;
  background: transparent url(images/bg-navigation-item-hover.png) no-repeat 50% 0;
}

/**
 * Secondary navigation
 */
ul.secondary-menu {
  margin: 0;
  padding: 18px 0 0;
  float: right; /* LTR */
  clear: right; /* LTR */
  position: relative;
  z-index: 4;
}

ul.secondary-menu li {
  margin: 0;
  padding: 0;
  float: left; /* LTR */
  background-image: none;
}

ul.secondary-menu li a,
ul.secondary-menu li a:link,
ul.secondary-menu li a:visited {
  display: block;
  margin: 0 1em;
  padding: .75em 0 0;
  color: #cde3f1;
  background: transparent;
}

ul.secondary-menu li a:hover,
ul.secondary-menu li a.active {
  color: #cde3f1;
  background: transparent;
}

/**
 * Local tasks
 */
ul.primary,
ul.primary li,
ul.secondary,
ul.secondary li {
  border: 0;
  background: none;
  margin: 0;
  padding: 0;
}

#tabs-wrapper {
  margin: 0 -26px 1em;
  padding: 0 26px;
  border-bottom: 1px solid #e9eff3;
  position: relative;
}
ul.primary {
  padding: 0.5em 0 10px;
  float: left; /* LTR */
}
ul.secondary {
  clear: both;
  text-align: left; /* LTR */
  border-bottom: 1px solid #e9eff3;
  margin: -0.2em -26px 1em;
  padding: 0 26px 0.6em;
}
h1.with-tabs {
  float: left; /* LTR */
  margin: 0 2em 0 0; /* LTR */
  padding: 0;
}

ul.primary li a,
ul.primary li.active a,
ul.primary li a:hover,
ul.primary li a:visited,
ul.secondary li a,
ul.secondary li.active a,
ul.secondary li a:hover,
ul.secondary li a:visited {
  border: 0;
  background: transparent;
  padding: 4px 1em;
  margin: 0 0 0 1px; /* LTR */
  height: auto;
  text-decoration: none;
  position: relative;
  top: -1px;
  display: inline-block;
}
ul.primary li.active a,
ul.primary li.active a:link,
ul.primary li.active a:visited,
ul.primary li a:hover,
ul.secondary li.active a,
ul.secondary li.active a:link,
ul.secondary li.active a:visited,
ul.secondary li a:hover {
  background: url(images/bg-tab.png) repeat-x 0 50%;
  color: #fff;
}
ul.primary li.active a,
ul.secondary li.active a {
  font-weight: bold;
}

/**
 * Nodes & comments
 */
.node {
  border-bottom: 1px solid #e9eff3;
  margin: 0 -16px 1.5em;
  padding: 1.5em 16px;
}

ul.links li,
ul.inline li {
  margin-left: 0;
  margin-right: 0;
  padding-left: 0; /* LTR */
  padding-right: 1em; /* LTR */
  background-image: none;
}

.node .links,
.comment .links {
  text-align: left; /* LTR */
  padding-left: 0; /* LTR */
}

.user-picture,
.comment .submitted {
  float: right; /* LTR */
  clear: right; /* LTR */
  padding-left: 1em; /* LTR */
}

.new {
  color: #ffae00;
  font-size: 0.92em;
  font-weight: bold;
  float: right; /* LTR */
}

.preview .node,
.preview .comment,
.node-sticky {
  margin: 0;
  padding: 0.5em 0;
  border: 0;
  background: 0;
}

.node-sticky {
  padding: 1em;
  background-color: #fff;
  border: 1px solid #e0e5fb;
  margin-bottom: 2em;
}

#comments {
  position: relative;
  top: -1px;
  border-bottom: 1px solid #e9eff3;
  margin: 1.5em -25px 0;
  padding: 0 25px;
}

#comments h2.comments {
  margin: 0 -25px;
  padding: .5em 25px;
  background: #fff url(images/gradient-inner.png) repeat-x 0 0;
}

.comment {
  margin: 0 -25px;
  padding: 1.5em 25px 1.5em;
  border-top: 1px solid #e9eff3;
}

.indented {
  margin-left: 25px; /* LTR */
}

.comment h3 a.active {
  color: #494949;
}

.node .content,
.comment .content {
  margin: 0.6em 0;
}

/**
 * Aggregator.module
 */
#aggregator {
  margin-top: 1em;
}
#aggregator .feed-item-title {
  font-size: 160%;
  line-height: 130%;
}
#aggregator .feed-item {
  border-bottom: 1px solid #e9eff3;
  margin: -1.5em -31px 1.75em;
  padding: 1.5em 31px;
}
#aggregator .feed-item-categories {
  font-size: 0.92em;
}
#aggregator .feed-item-meta {
  font-size: 0.92em;
  color: #898989;
}

/**
 * Color.module
 */
#palette .form-item {
  border: 1px solid #fff;
}
#palette .item-selected {
  background: #fff url(images/gradient-inner.png) repeat-x 0 0;
  border: 1px solid #d9eaf5;
}

/**
 * Menu.module
 */
tr.odd td.menu-disabled {
  background-color: #edf5fa;
}
tr.even td.menu-disabled {
  background-color: #fff;
}

/**
 * Poll.module
 */
.poll .bar {
  background: #fff url(images/bg-bar-white.png) repeat-x 0 0;
  border: solid #f0f0f0;
  border-width: 0 1px 1px;
}

.poll .bar .foreground {
  background: #71a7cc url(images/bg-bar.png) repeat-x 0 100%;
}

.poll .percent {
  font-size: .9em;
}

/**
 * Autocomplete.
 */
#autocomplete li {
  cursor: default;
  padding: 2px;
  margin: 0;
}

/**
 * Collapsible fieldsets
 */
fieldset {
  margin: 1em 0;
  padding: 1em;
  border: 1px solid #d9eaf5;
  background: #fff url(images/gradient-inner.png) repeat-x 0 0;
}

/* Targets IE 7. Fixes background image in field sets. */
*:first-child+html fieldset {
  padding: 0 1em 1em;
  background-position: 0 .75em;
  background-color: transparent;
}
*:first-child+html .text-format-wrapper .filter-wrapper {
  padding: 1.5em 0 1.5em;
  background-position: 0 0;
}

/* Keep the background position at 0 for filters and vertical tabs. */
*:first-child+html fieldset.filter-wrapper,
*:first-child+html fieldset.vertical-tabs-pane {
  background-position: 0 0;
}

*:first-child+html fieldset .fieldset-wrapper .fieldset-description {
  padding-top: 1em;
}

html.js fieldset.collapsed {
  background: transparent;
  padding-top: 0;
  padding-bottom: .6em;
}

html.js fieldset.collapsible .fieldset-legend {
  padding-left: 2em; /* LTR */
  background: url(images/menu-expanded.gif) no-repeat 0% 50%; /* LTR */
}
html.js fieldset.collapsed .fieldset-legend {
  background: url(images/menu-collapsed.gif) no-repeat 0% 50%; /* LTR */
}

.fieldset-legend span.summary {
  color: #898989;
}

/**
 * Vertical tabs.
 */
div.vertical-tabs {
  border-color: #d9eaf5;
}

div.vertical-tabs .vertical-tabs-panes fieldset.vertical-tabs-pane {
  padding: 0.5em 1em;
}

div.vertical-tabs ul.vertical-tabs-list {
  border-color: #d9eaf5;
}

div.vertical-tabs ul.vertical-tabs-list li {
  background-color: #edf5fa;
  border-color: #d9eaf5;
}

div.vertical-tabs ul.vertical-tabs-list li strong {
  font-weight: normal;
}

div.vertical-tabs ul.vertical-tabs-list li.selected strong {
  font-weight: bold;
}

div.vertical-tabs ul.vertical-tabs-list li.selected small {
  color: #898989;
}

div.vertical-tabs ul.vertical-tabs-list li.selected {
  background: #fff repeat-x 0 0;
}

div.vertical-tabs ul.vertical-tabs-list li.selected.first {
  background-image: url(images/gradient-inner.png);
}

div.vertical-tabs ul.vertical-tabs-list li.selected a strong {
  color: #494949;
}

/**
 * Syndication icons and block
 */
#block-node-syndicate h2 {
  float: left; /* LTR */
  padding-right: 20px; /* LTR */
}

#block-node-syndicate img,
.feed-icon {
  float: right; /* LTR */
  padding-top: 4px;
}

#block-node-syndicate .content {
  clear: right; /* LTR */
}

/**
 * Login Block
 */
#user-login-form {
  text-align: center;
}
#user-login-form ul {
  text-align: left; /* LTR */
}
#user-login .openid-links {
  padding: 0;
}
#user-login-form .openid-links {
  padding-left: 0.5em; /* LTR */
}
#user-login-form .openid-links li.user-link {
  padding-left: 1em; /* LTR */
}

/**
 * User profiles.
 */
.profile {
  margin-top: 1.5em;
}
.profile h3 {
  border-bottom: 0;
  margin-bottom: 1em;
}
.profile dl {
  margin: 0;
}
.profile dt {
  font-weight: normal;
  color: #898989;
  font-size: 0.92em;
  line-height: 1.3em;
  margin-top: 1.4em;
  margin-bottom: 0.45em;
}
.profile dd {
  margin-bottom: 1.6em;
}

/**
 * Admin Styles
 */
div.admin-panel,
div.admin-panel .description,
div.admin-panel .body,
div.admin,
div.admin .left,
div.admin .right,
div.admin .expert-link,
div.item-list,
.menu {
  margin: 0;
  padding: 0;
}

div.admin .left {
  float: left; /* LTR */
  width: 48%;
}
div.admin .right {
  float: right; /* LTR */
  width: 48%;
}

div.admin-panel {
  background: #fff url(images/gradient-inner.png) repeat-x 0 0;
  padding: 1em 1em 1.5em;
}
div.admin-panel .description {
  margin-bottom: 1.5em;
}
div.admin-panel dl {
  margin: 0;
}
div.admin-panel dd {
  color: #898989;
  font-size: 0.92em;
  line-height: 1.3em;
  margin-top: -.2em;
  margin-bottom: .65em;
}

table.system-status-report th {
  border-color: #d3e7f4;
}

#autocomplete li.selected,
tr.selected td,
tr.selected td.active {
  background: #027ac6;
  color: #fff;
}

tr.selected td a:link,
tr.selected td a:visited,
tr.selected td a:active {
  color: #d3e7f4;
}

tr.taxonomy-term-preview {
  opacity: 0.5;
}

tr.taxonomy-term-divider-top {
  border-bottom: none;
}

tr.taxonomy-term-divider-bottom {
  border-top: 1px dotted #CCC;
}

.theme-selector h3 {
  margin: 10px 0;
}
.theme-selector .operations li {
  list-style-image: none;
}

/**
 * CSS support
 */

/*******************************************************************
 * Color Module: Don't touch                                       *
 *******************************************************************/

/**
 * Generic elements.
 */
.preview {
  background-color: #fcfce8;
  border: 1px solid #e5e58f;
}

.form-item input.error,
.form-item textarea.error {
  border: 1px solid #c52020;
  color: #363636;
}

/**
 * dblog.module
 */
tr.dblog-user {
  background-color: #fcf9e5;
}

tr.dblog-user td.active {
  background-color: #fbf5cf;
}

tr.dblog-content {
  background-color: #fefefe;
}

tr.dblog-content td.active {
  background-color: #f5f5f5;
}

tr.dblog-warning {
  background-color: #fdf5e6;
}

tr.dblog-warning td.active {
  background-color: #fdf2de;
}

tr.dblog-error {
  background-color: #fbe4e4;
}

tr.dblog-error td.active {
  background-color: #fbdbdb;
}
tr.dblog-page-not-found,
tr.dblog-access-denied {
  background: #d7ffd7;
}
tr.dblog-page-not-found td.active,
tr.dblog-access-denied td.active {
  background: #c7eec7;
}

/**
 * Status report colors.
 */
table.system-status-report tr.error,
table.system-status-report tr.error th {
  background-color: #fcc;
  border-color: #ebb;
  color: #200;
}
table.system-status-report tr.warning,
table.system-status-report tr.warning th {
  background-color: #ffd;
  border-color: #eeb;
}
table.system-status-report tr.ok,
table.system-status-report tr.ok th {
  background-color: #dfd;
  border-color: #beb;
}
