
.block {
  zoom: 1;
}
#password-strength-text {
  margin-top: 0;
}
fieldset legend {
  left: -8px;
  padding: 0;
}
#footer-wrapper #footer .block {
  height: 100%;
}
.comment .attribution {
  display: inline-block;
  position: relative;
  float: left; /* LTR */
  overflow: hidden;
}
.comment .comment-text {
  display: inline-block;
  width: auto;
}
#search-block-form input.form-submit,
#search-form input.form-submit {
  text-transform: capitalize; /* Trigger text indent. */
  height: 26px;
}
.meta {
  margin-bottom: 10px;
}
.region-header .form-required {
  color: #eee;
}
#search-block-form input.form-submit:hover,
#search-block-form input.form-submit:focus,
#search-form input.form-submit:hover,
#search-form input.form-submit:focus {
  background-position: center -25px;
}
.contact-form #edit-message {
  width: 75%;
}
.contact-form .resizable-textarea .grippie {
  width: 76.3%;
}
#footer li {
  float: left; /* LTR */
}
#footer-wrapper {
  color: #c0c0c0;
}
#footer-wrapper a {
  color: #fcfcfc;
}
#footer-wrapper a:hover,
#footer-wrapper a:focus {
  color: #fefefe;
}
.node-teaser {
  margin-top: 10px;
}
