.at-panel .region,.panel-row {display:inline;float:left}
.inset-wrapper {float:left;width:75%}
.five-5x20 > .row-1 {width:40%}
.five-5x20 > .row-2 {width:60%}
.three-50-25-25 > .region,.three-25-50-25 > .region,.three-25-25-50 > .region,.three-inset-right > .region-three-inset-right-sidebar,.three-inset-left > .region-three-inset-left-sidebar {width:25%}
.two-50 > .region,.two-brick > .panel-row > .region,.three-50-25-25 > .region-three-50-25-25-first,.three-25-50-25 > .region-three-25-50-25-second,.three-25-25-50 > .region-three-25-25-50-third,.four-4x25 > .panel-row > .region,.four-4x25 > .row-1,.four-4x25 > .row-2,.five-5x20 > .row-1 .region,.six-6x16 > .panel-row > .region {width:50%}
.three-3x33 > .region,.two-33-66 > .region,.two-66-33 > .region,.three-inset-right > .inset-wrapper > .region-three-inset-right-inset,.three-inset-left > .inset-wrapper > .region-three-inset-left-inset,.five-5x20 > .row-2 > .region,.six-6x16 > .panel-row {width:33.333333%}
.two-33-66 > .region-two-33-66-second,.two-66-33 > .region-two-66-33-first,.three-inset-left > .inset-wrapper > .region-three-inset-left-middle,.three-inset-right > .inset-wrapper > .region-three-inset-right-middle {width:66.666666%}
.three-inset-right .region,.three-inset-right .inset-wrapper {float:right}
.two-brick > .panel-row {float:none}