/**
 * @file
 * Styles used by the Update Manager module.
 */

.update .project {
  font-weight: bold;
  font-size: 110%;
  padding-left: .25em; /* LTR */
  height: 22px;
}

.update .version-status {
  float: right; /* LTR */
  padding-right: 10px; /* LTR */
  font-size: 110%;
  height: 20px;
}

.update .version-status .icon {
  padding-left: .5em; /* LTR */
}

.update .version-date {
  white-space: nowrap;
}

.update .info {
  margin: 0;
  padding: 1em 1em .25em 1em;
}

.update tr.even,
.update tr.odd {
  border: none;
}

.update tr td {
  border-top: 1px solid #ccc;
  border-bottom: 1px solid #ccc;
}

.update tr.error {
  background: #fcc;
}

.update tr.error .version-recommended {
  background: #fdd;
}

.update tr.ok {
  background: #dfd;
}

.update tr.warning {
  background: #ffd;
}

.update tr.warning .version-recommended {
  background: #ffe;
}

.current-version,
.new-version {
  direction: ltr; /* Note: version numbers should always be LTR. */
}

.update tr.unknown {
  background: #ddd;
}

table.update,
.update table.version {
  width: 100%;
  margin-top: .5em;
  border: none;
}

.update table.version tbody {
  border: none;
}

.update table.version tr,
.update table.version td {
  line-height: .9em;
  padding: 0;
  margin: 0;
  border: none;
  background: none;
}

.update table.version .version-title {
  padding-left: 1em; /* LTR */
  width: 14em;
}

.update table.version .version-details {
  padding-right: .5em; /* LTR */
}

.update table.version .version-links {
  text-align: right; /* LTR */
  padding-right: 1em; /* LTR */
}

.update table.version-security .version-title {
  color: #970F00;
}

.update table.version-recommended-strong .version-title {
  font-weight: bold;
}

.update .security-error {
  font-weight: bold;
  color: #970F00;
}

.update .check-manually {
  padding-left: 1em; /* LTR */
}

.update-major-version-warning {
  color: #ff0000;
}

table tbody tr.update-security,
table tbody tr.update-unsupported {
  background: #fcc;
}

th.update-project-name {
  width: 50%;
}

