/* Image Alignment */
.field-type-image {width:auto}
.field-type-image img {display:block}
.field-type-image figure {overflow:hidden}
.field-type-image figure,.field-type-image figcaption {max-width:100%}
.field-type-image figcaption {text-align:left}
.field-type-image .field-label {margin-bottom:5px}

/* IE workarounds for less than IE8, this does not fix much, but at least its not totally destroyed */
.lt-ie8 .field-type-image img,
.lt-ie8 .field-type-image figure,
.lt-ie8 .field-type-image figcaption {max-width: none !important}

/* Left */
.image-left .field-type-image.field-label-above .field-label,
.image-left .field-type-image figure,
.ia-l .field-type-image.field-label-above .field-label,
.iat-l .field-type-image.field-label-above .field-label,
.ia-l .field-type-image figure,
.iat-l .field-type-image figure {float:left;clear:left}

/* Right */
.image-right .field-type-image.field-label-above .field-label,
.image-right .field-type-image figure,
.ia-r .field-type-image.field-label-above .field-label,
.iat-r .field-type-image.field-label-above .field-label,
.ia-r .field-type-image figure,
.iat-r .field-type-image figure {float:right;clear:right}

/* Center */
.image-center .field-type-image figcaption,
.image-center .field-type-image img,
.ia-c .field-type-image figcaption,
.iat-c .field-type-image figcaption,
.ia-c .field-type-image img,
.iat-c .field-type-image img {text-align:center;margin-left:auto;margin-right:auto}
