/*
 *   Important Information about this CSS File
 *
 * - Do not delete or rename this file, if you do not use it leave it blank (delete
 *   everything) and the file will be skipped when you enable Production Mode in
 *   the Global theme settings.
 *
 * - Read the _README file in this directory, it contains useful help and other information.
 */
/* Increase the body font size on small devices */
html {
  font-size: 81.3%;
}
