
/* Field display */
.field .field-label {
  font-weight: bold;
}
.field-label-inline .field-label,
.field-label-inline .field-items {
  float:left; /*LTR*/
}

/* Form display */
form .field-multiple-table {
  margin: 0;
}
form .field-multiple-table th.field-label {
  padding-left: 0; /*LTR*/
}
form .field-multiple-table td.field-multiple-drag {
  width: 30px;
  padding-right: 0; /*LTR*/
}
form .field-multiple-table td.field-multiple-drag a.tabledrag-handle {
  padding-right: .5em; /*LTR*/
}

form .field-add-more-submit {
  margin: .5em 0 0;
}
