
#toolbar,
#toolbar * {
  text-align: right;
}
#toolbar ul li {
  float: right;
}
#toolbar ul li a {
  display: inline-block;
  float: none;
  zoom: 1;
}
#toolbar div.toolbar-menu {
  padding: 5px 50px 5px 50px;
}
#toolbar-user {
  float: left;
}
#toolbar ul#toolbar-user li {
  float: none;
  display: inline;
}
#toolbar-menu {
  float: none;
}
#toolbar-home {
  float: right;
}
#toolbar ul li.home a {
  position: absolute;
  right: 10px;
}
#toolbar div.toolbar-menu a.toggle {
  left: 10px;
  right: auto;
}
* html #toolbar {
  left: 0;
  padding-left: 0;
}
