
/**
 * @file
 * RTL styles for menus and navigation markup.
 */

ul.menu {
  text-align:right;
}
ul.menu li {
  margin: 0 0.5em 0 0;
}
ul li.collapsed {
  list-style-image: url(../../misc/menu-collapsed-rtl.png);
}
li.expanded,
li.collapsed,
li.leaf {
  padding: 0.2em 0 0 0.5em;
}

/**
 * Markup generated by theme_menu_local_tasks().
 */
ul.primary {
  padding: 0 1em 0 0;
}
ul.primary li a {
  margin-right: 5px;
  margin-left: 0.5em;
}
ul.secondary li {
  border-left: 1px solid #ccc;
  border-right: none;
  display: inline;
  padding: 0 1em;
}
