
/**
 * Reset CSS styles.
 *
 * Based on Eric Meyer's "Reset CSS 1.0" tool from
 * http://meyerweb.com/eric/tools/css/reset
 */

html,
body,
div,
span,
applet,
object,
iframe,
h1,
h2,
h3,
h4,
h5,
h6,
p,
blockquote,
pre,
a,
abbr,
acronym,
address,
big,
cite,
code,
del,
dfn,
em,
font,
img,
ins,
kbd,
q,
s,
samp,
small,
strike,
strong,
sub,
sup,
tt,
var,
b,
u,
i,
center,
dl,
dt,
dd,
ol,
ul,
li,
fieldset,
form,
input,
select,
textarea,
label,
legend,
table,
caption,
tbody,
tfoot,
thead,
tr,
th,
td,
/* Drupal: system-menus.css */
td.menu-disabled,
ul.links,
ul.links.inline,
ul.links li,
.block ul,
/* Drupal: admin.css */
div.admin,
/* Drupal: system.css */
tr.even,
tr.odd,
tr.drag,
tbody,
tbody th,
thead th,
.breadcrumb,
.item-list .icon,
.item-list .title,
.item-list ul,
.item-list ul li,
ol.task-list li.active,
.form-item,
tr.odd .form-item,
tr.even .form-item,
.form-item .description,
.form-item label,
.form-item label.option,
.form-checkboxes,
.form-radios,
.form-checkboxes .form-item,
.form-radios .form-item,
.marker,
.form-required,
.more-link,
.more-help-link,
.item-list .pager,
.item-list .pager li,
.pager-current,
.tips,
ul.primary,
ul.primary li,
ul.primary li a,
ul.primary li.active a,
ul.primary li a:hover,
ul.secondary,
ul.secondary li,
ul.secondary a,
ul.secondary a.active,
.resizable-textarea {
  margin: 0;
  padding: 0;
  border: 0;
  vertical-align: baseline;
}
/* Drupal: system-menus.css */
ul.links,
ul.links.inline,
ul.links li,
.block ul,
ol,
ul,
.item-list ul,
.item-list ul li {
  list-style: none;
}
blockquote,
q {
  quotes: none;
}
blockquote:before,
blockquote:after,
q:before,
q:after {
  content: '';
  content: none;
}

/* Remember to highlight inserts somehow! */
ins {
  text-decoration: none;
}
del {
  text-decoration: line-through;
}

/* Tables still need 'cellspacing="0"' in the markup. */
table {
  border-collapse: collapse;
  border-spacing: 0;
}

/**
 * Font reset.
 *
 * Specifically targets form elements which browsers often times give
 * special treatment.
 */
input,
select,
textarea {
  font-family: "Helvetica Neue", Helvetica, Arial, sans-serif;
}
textarea {
  font-size: 1em;
  line-height: 1.538em;
}
/**
 * Markup free clearing.
 *
 * Consider adding your own selectors to this instead of finding ways
 * to sneak the clearfix class into Drupal's markup.
 * From http://perishablepress.com/press/2009/12/06/new-clearfix-hack
 */
ul.links:after,
div.admin-panel .body:after,
.clearfix:after {
  content: ".";
  display: block;
  height: 0;
  clear: both;
  visibility: hidden;
}

/* Exclude inline links from clearfix behavior */
ul.inline:after {
  content: "";
  display: none;
  clear: none;
}
/* IE6 */
* html .form-item,
* html ul.links,
* html div.admin-panel .body,
* html .clearfix {
  height: 1%;
}
/* IE7 */
*:first-child + html .form-item,
*:first-child + html ul.links,
*:first-child + html div.admin-panel .body,
*:first-child + html .clearfix {
  min-height: 1%;
}
