
body.maintenance-page {
  background-color: #fff;
  color: #000;
}
.maintenance-page #page-wrapper {
  background: #fff;
  margin-left: auto;
  margin-right: auto;
  min-width: 0;
  min-height: 0;
  width: 800px;
  border: 1px solid #ddd;
  margin-top: 40px;
}
.maintenance-page #page {
  margin: 20px 40px 40px;
}
.maintenance-page #main-wrapper {
  min-height: inherit;
}
.maintenance-page #header,
.maintenance-page #messages,
.maintenance-page #main {
  width: auto;
}
.maintenance-page #header div.section,
.maintenance-page #main {
  width: 700px;
}
.maintenance-page #main {
  margin: 0;
}
.maintenance-page #content .section {
  padding: 0 0 0 10px;
}
.maintenance-page #header {
  background-color: #fff;
  background-image: none;
}
.maintenance-page #name-and-slogan {
  margin-bottom: 50px;
  margin-left: 0;
  padding-top: 20px;
  font-size: 90%;
}
.maintenance-page #name-and-slogan,
.maintenance-page #name-and-slogan a,
.maintenance-page #name-and-slogan a:hover,
.maintenance-page #name-and-slogan a:hover {
  color: #777;
}
.maintenance-page  h1#page-title {
  line-height: 1em;
  margin-top: 0;
}
.maintenance-page #messages {
  padding: 0;
  margin-top: 30px;
}
.maintenance-page #messages div.messages {
  margin: 0;
}
.maintenance-page #messages div.section {
  padding: 0;
  width: auto;
}
