
/**
 * @file
 * RTL styles for common markup.
 */

/**
 * HTML elements.
 */
th {
  text-align: right;
  padding-left: 1em;
  padding-right: 0;
}

/**
 * Markup generated by theme_item_list().
 */
.item-list ul li {
  margin: 0 1.5em 0.25em 0;
}

/**
 * Markup generated by theme_more_link().
 */
.more-link {
  text-align: left;
}

/**
 * Markup generated by theme_more_help_link().
 */
.more-help-link {
  text-align: left;
}
.more-help-link a {
  background-position: 100% 50%;
  padding: 1px 20px 1px 0;
}

/**
 * Collapsible fieldsets.
 */
html.js fieldset.collapsible .fieldset-legend {
  background-position: 98% 75%;
  padding-left: 0;
  padding-right: 15px;
}
html.js fieldset.collapsed .fieldset-legend {
  background-image: url(../../misc/menu-collapsed-rtl.png);
  background-position: 98% 50%;
}

