#panels-dnd-main .region {margin-bottom: 5px}
#panels-dnd-main .region-two-50-first {float: left; width: 49.75%}
#panels-dnd-main .region-two-50-second {float: right; width: 49.75%}
#panels-dnd-main .region-two-50-bottom {clear: both}
